

#include "fapi.h"
#include "fdetypes.h"
#include "futils.h"
#include "fstrings.h"
#include "fmetrics.h"
#include "fmemory.h"
#include "fcodes.h"

VoidT ABCM_Sample()
{
  F_ObjHandleT docId,
    docId2,
    flowId,
    elemId;

  UCharT msg[512];
  
  IntT returnVal;

  UIntT i;

  //Let's store the ID of the active document, which MUST BE the
  //External_Calls_Sample.fm file for this whole function to work
  docId = F_ApiGetId(0, FV_SessionId, FP_ActiveDoc);
  if(!docId) return;
  
  //Message
  F_ApiAlert("Preparing to color.  Click OK to set the active scheme...",
    FF_ALERT_CONTINUE_WARN);

  //Set the active coloring scheme.  Let's do a bogus scheme first, just as
  //an example of a call failure
  returnVal = F_ApiCallClient("ABCM", "SetScheme---C---BogusCategory---BogusScheme");

  //send a message about the failure.  Should indicate a return value of 106
  F_Sprintf(msg, "The command to set the scheme failed, because the category\
 and scheme name were bogus. The call returned the following integer:  %d\n\n\
 Click OK to try again.",
    returnVal);
  
  F_ApiAlert(msg, FF_ALERT_CONTINUE_WARN);
  
  //Set the active coloring scheme for real this time
  returnVal = F_ApiCallClient("ABCM", "SetScheme---C---Samples---External_Call_Sample");

  //Evaluate the response.  We're going to back out if something went wrong.
  if(returnVal == True) 
    F_ApiAlert("The scheme was set successfully. Click OK to launch the coloring.", 
      FF_ALERT_CONTINUE_WARN);

  else
  {
    F_Sprintf(msg, "The command failed, returning the following integer:  %d\n\n\
 Your main settings do not include the factory samples necessary for this test.\
 The test will abort.", returnVal);
  
    F_ApiAlert(msg, FF_ALERT_CONTINUE_WARN);

    return;
  }

  //Launch the coloring.
  returnVal = F_ApiCallClient("ABCM", "ColorFile---External_Calls_Sample.fm---False");

  //Let's make sure it worked, before continuing.  If you forgot to open the file 
  //beforehand, this would have caused it to fail.
  if(returnVal == True) 
    F_ApiAlert("The file was successfully colored. Click OK to launch the filtering.", 
      FF_ALERT_CONTINUE_WARN);

  else
  {
    F_Sprintf(msg, "The coloring failed. Did you forget to open the file?", 
      returnVal);
  
    F_ApiAlert(msg, FF_ALERT_CONTINUE_WARN);

    return;
  }

  //Set the filtering scheme
  returnVal = F_ApiCallClient("ABCM", "SetScheme---F---Samples---External_Call_Sample");

  //Set the filter type to duplicate file
  returnVal = F_ApiCallClient("ABCM", "SetParm---Filter_Type---D");

  //Set the "save files first" option to off
  returnVal = F_ApiCallClient("ABCM", "SetParm---Filter_SaveFirst---0");

  //Filter the document
  returnVal = F_ApiCallClient("ABCM", "FilterFile---External_Calls_Sample.fm---False");

  //If it filtered successfully, we should have received an object ID back of the
  //filtered duplicate.  Just as a sample of what we can do with it, let's color the
  //filtered duplicate now.
  
  //Let's save the document ID first.
  if(returnVal > 1000)
  {
    docId2 = (F_ObjHandleT)returnVal;
    F_ApiAlert("The file was successfully filtered. Click OK to color the new file.", 
      FF_ALERT_CONTINUE_WARN);
  }
  else
  {
    F_ApiAlert("Something went wrong.  Aborting the test.", FF_ALERT_CONTINUE_WARN);
    return;
  }

  F_Sprintf(msg, "ColorFile---%d---False", docId2);
  returnVal = F_ApiCallClient("ABCM", msg);
  

  //Get ready to close the filtered duplicate
  F_ApiAlert("The duplicate file was colored. Click OK to close it.", 
    FF_ALERT_CONTINUE_WARN);

  F_ApiClose(docId2, FF_CLOSE_MODIFIED);

  //Now let's do a source-file filter
  
  F_ApiAlert("The duplicate file was closed. Now click OK to perform a\
 source-file filter on the original document.", FF_ALERT_CONTINUE_WARN);

  //Set the filter type to source file
  returnVal = F_ApiCallClient("ABCM", "SetParm---Filter_Type---S");

  //Filter the document
  returnVal = F_ApiCallClient("ABCM", "FilterFile---External_Calls_Sample.fm---False");
    
  //Report
  F_ApiAlert("The file was filtered. Click OK to restore it.", 
    FF_ALERT_CONTINUE_WARN);

  //Restore the filtered file.
  returnVal = F_ApiCallClient("ABCM", "RestoreFile---External_Calls_Sample.fm");

  //and let's remove the coloring
  F_ApiSimpleImportElementDefs(docId, docId, 
    FF_IED_REMOVE_OVERRIDES | FF_IED_REMOVE_BOOK_INFO);

  
  //Just for kicks, let's do some element-level functions.

  //Let's get the ID of the first Section element, 
  //the one tagged for Product A.
  flowId = F_ApiGetId(FV_SessionId, docId, FP_MainFlowInDoc);
  elemId = F_ApiGetId(docId, flowId, FP_HighestLevelElement);
  elemId = F_ApiGetId(docId, elemId, FP_FirstChildElement);
  for(i = 0; i < 5; i++)
    elemId = F_ApiGetId(docId, elemId, FP_NextSiblingElement);

  //Prompt
  F_ApiAlert("The code has retrieved the ID of the first Section element.\
 Click OK to color it.", FF_ALERT_CONTINUE_WARN);

  //Color the first Section element
  F_Sprintf(msg, "ColorElement---%d---%d---True---False", docId, elemId);
  returnVal = F_ApiCallClient("ABCM", msg);

  //Prompt
  F_ApiAlert("The element has been colored. Click OK to filter check it.",
    FF_ALERT_CONTINUE_WARN);

  //Filter-check the first Section element.  It should return a value of
  //zero, meaning that the element should not be filtered out.  In other
  //words, the active coloring scheme allows it to stay.
  F_Sprintf(msg, "FilterCheckElement---%d---%d", docId, elemId);
  returnVal = F_ApiCallClient("ABCM", msg);

  F_Sprintf(msg, "The filter check returned: %d.\n\n\
0 = Keep\n1 = Filter out", returnVal);
  F_ApiAlert(msg, FF_ALERT_CONTINUE_WARN);


  //Do the same thing for the second Section element.  This one should
  //return a flag to filter it out, like it was filtered out when we
  //ran the full document filter earlier.

  //Get the ID of the second Section element
  elemId = F_ApiGetId(docId, elemId, FP_NextSiblingElement);

  //Prompt
  F_ApiAlert("The code has retrieved the ID of the second Section element.\
 Click OK to color it.", FF_ALERT_CONTINUE_WARN);

  //Color the second Section element
  F_Sprintf(msg, "ColorElement---%d---%d---True---False", docId, elemId);
  returnVal = F_ApiCallClient("ABCM", msg);

  //Prompt
  F_ApiAlert("The element has been colored. Click OK to filter check it.",
    FF_ALERT_CONTINUE_WARN);

  //Filter-check the second Section element.  It should return a value of
  //one, meaning that the element should be filtered out.  In other words,
  //it is content to be hidden.
  F_Sprintf(msg, "FilterCheckElement---%d---%d", docId, elemId);
  returnVal = F_ApiCallClient("ABCM", msg);

  F_Sprintf(msg, "The filter check returned: %d.\n\n\
0 = Keep\n1 = Filter out", returnVal);
  F_ApiAlert(msg, FF_ALERT_CONTINUE_WARN);

  
  F_ApiAlert("All done!", FF_ALERT_CONTINUE_WARN);
}

 


